@@ -12,6 +12,11 @@
 #    under the License.
 #
 
+from neutron.db import dns_db
+from neutron.db import extraroute_db
+from neutron.db import l3_gwmode_db
+from neutron.db.models import l3 as l3_models
+from neutron.quota import resource_registry
 from neutron_lib.api.definitions import external_net
 from neutron_lib.api.definitions import portbindings
 from neutron_lib.api.definitions import provider_net as pnet
@@ -27,18 +32,13 @@ from neutron_lib.services import base as service_base
 from oslo_log import log
 from oslo_utils import excutils
 
-from neutron.db import dns_db
-from neutron.db import extraroute_db
-from neutron.db import l3_gwmode_db
-from neutron.db.models import l3 as l3_models
-from neutron.quota import resource_registry
-
 from networking_ovn.common import constants as ovn_const
 from networking_ovn.common import extensions
 from networking_ovn.common import ovn_client
 from networking_ovn.common import utils
 from networking_ovn.db import revision as db_rev
 from networking_ovn.l3 import l3_ovn_scheduler
+from networking_ovn.ovsdb import impl_idl_ovn
 
 
 LOG = log.getLogger(__name__)
@@ -55,16 +55,17 @@ class OVNL3RouterPlugin(service_base.ServicePluginBase,
     router and floatingip resources and manages associated
     request/response.
     """
-    supported_extension_aliases = \
-        extensions.ML2_SUPPORTED_API_EXTENSIONS_OVN_L3
+    supported_extension_aliases = (
+        extensions.ML2_SUPPORTED_API_EXTENSIONS_OVN_L3)
 
     @resource_registry.tracked_resources(router=l3_models.Router,
                                          floatingip=l3_models.FloatingIP)
     def __init__(self):
         LOG.info("Starting OVNL3RouterPlugin")
         super(OVNL3RouterPlugin, self).__init__()
+        self._nb_ovn_idl = None
+        self._sb_ovn_idl = None
         self._plugin_property = None
-        self._mech = None
         self._ovn_client_inst = None
         self.scheduler = l3_ovn_scheduler.get_scheduler()
         self._register_precommit_callbacks()
@@ -86,11 +87,19 @@ class OVNL3RouterPlugin(service_base.ServicePluginBase,
 
     @property
     def _ovn(self):
-        return self._plugin_driver.nb_ovn
+        if self._nb_ovn_idl is None:
+            LOG.info("Getting OvsdbNbOvnIdl")
+            conn = impl_idl_ovn.get_connection(impl_idl_ovn.OvsdbNbOvnIdl)
+            self._nb_ovn_idl = impl_idl_ovn.OvsdbNbOvnIdl(conn)
+        return self._nb_ovn_idl
 
     @property
     def _sb_ovn(self):
-        return self._plugin_driver.sb_ovn
+        if self._sb_ovn_idl is None:
+            LOG.info("Getting OvsdbSbOvnIdl")
+            conn = impl_idl_ovn.get_connection(impl_idl_ovn.OvsdbSbOvnIdl)
+            self._sb_ovn_idl = impl_idl_ovn.OvsdbSbOvnIdl(conn)
+        return self._sb_ovn_idl
 
     @property
     def _plugin(self):
@@ -98,12 +107,6 @@ class OVNL3RouterPlugin(service_base.ServicePluginBase,
             self._plugin_property = directory.get_plugin()
         return self._plugin_property
 
-    @property
-    def _plugin_driver(self):
-        if self._mech is None:
-            self._mech = self._plugin.mechanism_manager.mech_drivers['ovn'].obj
-        return self._mech
-
     def get_plugin_type(self):
         return plugin_constants.L3
 
@@ -193,9 +196,9 @@ class OVNL3RouterPlugin(service_base.ServicePluginBase,
         return router_interface_info
 
     def remove_router_interface(self, context, router_id, interface_info):
-        router_interface_info = \
+        router_interface_info = (
             super(OVNL3RouterPlugin, self).remove_router_interface(
-                context, router_id, interface_info)
+                context, router_id, interface_info))
         try:
             port_id = router_interface_info['port_id']
             subnet_ids = router_interface_info.get('subnet_ids')
@@ -276,12 +279,12 @@ class OVNL3RouterPlugin(service_base.ServicePluginBase,
             return port_physnet_dict
         context = n_context.get_admin_context()
         for net in l3plugin._plugin.get_networks(
-            context, {external_net.EXTERNAL: [True]}):
+                context, {external_net.EXTERNAL: [True]}):
             if net.get(pnet.NETWORK_TYPE) in [n_const.TYPE_FLAT,
                                               n_const.TYPE_VLAN]:
                 net_physnet_dict[net['id']] = net.get(pnet.PHYSICAL_NETWORK)
         for port in l3plugin._plugin.get_ports(context, filters={
-            'device_owner': [n_const.DEVICE_OWNER_ROUTER_GW]}):
+                'device_owner': [n_const.DEVICE_OWNER_ROUTER_GW]}):
             port_physnet_dict[port['id']] = net_physnet_dict.get(
                 port['network_id'])
         return port_physnet_dict
@@ -307,48 +310,24 @@ class OVNL3RouterPlugin(service_base.ServicePluginBase,
             if port['status'] != status:
                 self._plugin.update_port_status(context, port['id'], status)
 
-    def schedule_unhosted_gateways(self, event_from_chassis=None):
-        # GW ports and its physnets.
+    def schedule_unhosted_gateways(self):
         port_physnet_dict = self._get_gateway_port_physnet_mapping()
-        # Filter out unwanted ports in case of event.
-        if event_from_chassis:
-            gw_chassis = self._ovn.get_chassis_gateways(
-                chassis_name=event_from_chassis)
-            if not gw_chassis:
-                return
-            ports_impacted = []
-            for gwc in gw_chassis:
-                try:
-                    ports_impacted.append(utils.get_port_id_from_gwc_row(gwc))
-                except AttributeError:
-                    # Malformed GWC format.
-                    pass
-            port_physnet_dict = {
-                k: v
-                for k, v in port_physnet_dict.items()
-                if k in ports_impacted}
-        if not port_physnet_dict:
-            return
-        # All chassis with physnets configured.
-        chassis_with_physnets = self._sb_ovn.get_chassis_and_physnets()
-        # All chassis with enable_as_gw_chassis set
-        all_gw_chassis = self._sb_ovn.get_gateway_chassis_from_cms_options()
+        chassis_physnets = self._sb_ovn.get_chassis_and_physnets()
+        cms = self._sb_ovn.get_gateway_chassis_from_cms_options()
         unhosted_gateways = self._ovn.get_unhosted_gateways(
-            port_physnet_dict, chassis_with_physnets,
-            all_gw_chassis)
+            port_physnet_dict, chassis_physnets, cms)
         for g_name in unhosted_gateways:
-            physnet = port_physnet_dict.get(g_name[len(ovn_const.LRP_PREFIX):])
+            physnet = port_physnet_dict.get(g_name[len('lrp-'):])
             # Remove any invalid gateway chassis from the list, otherwise
             # we can have a situation where all existing_chassis are invalid
             existing_chassis = self._ovn.get_gateway_chassis_binding(g_name)
             master = existing_chassis[0] if existing_chassis else None
             existing_chassis = self.scheduler.filter_existing_chassis(
-                nb_idl=self._ovn, gw_chassis=all_gw_chassis,
-                physnet=physnet, chassis_physnets=chassis_with_physnets,
+                nb_idl=self._ovn, gw_chassis=cms,
+                physnet=physnet, chassis_physnets=chassis_physnets,
                 existing_chassis=existing_chassis)
             candidates = self._ovn_client.get_candidates_for_scheduling(
-                physnet, cms=all_gw_chassis,
-                chassis_physnets=chassis_with_physnets)
+                physnet, cms=cms, chassis_physnets=chassis_physnets)
             chassis = self.scheduler.select(
                 self._ovn, self._sb_ovn, g_name, candidates=candidates,
                 existing_chassis=existing_chassis)
