@@ -12,13 +12,19 @@
 #    under the License.
 #
 
+import atexit
 import copy
 import functools
 import operator
+import signal
 import threading
 import types
 import uuid
 
+from neutron.common import utils as n_utils
+from neutron.db import provisioning_blocks
+from neutron.plugins.ml2 import db as ml2_db
+from neutron.services.segments import db as segment_service_db
 from neutron_lib.api.definitions import portbindings
 from neutron_lib.callbacks import events
 from neutron_lib.callbacks import registry
@@ -29,16 +35,12 @@ from neutron_lib import exceptions as n_exc
 from neutron_lib.plugins import directory
 from neutron_lib.plugins.ml2 import api
 from neutron_lib.services.qos import constants as qos_consts
+from neutron_lib.services.trunk import constants as trunk_consts
 from oslo_config import cfg
 from oslo_db import exception as os_db_exc
 from oslo_log import log
 from oslo_utils import timeutils
 
-from neutron.common import utils as n_utils
-from neutron.db import provisioning_blocks
-from neutron.plugins.ml2 import db as ml2_db
-from neutron.services.segments import db as segment_service_db
-
 from networking_ovn._i18n import _
 from networking_ovn.agent import stats
 from networking_ovn.common import acl as ovn_acl
@@ -48,6 +50,7 @@ from networking_ovn.common import exceptions as ovn_exc
 from networking_ovn.common import maintenance
 from networking_ovn.common import ovn_client
 from networking_ovn.common import utils
+from networking_ovn.db import hash_ring as db_hash_ring
 from networking_ovn.db import revision as db_rev
 from networking_ovn.ml2 import qos_driver
 from networking_ovn.ml2 import trunk_driver
@@ -104,7 +107,13 @@ class OVNMechanismDriver(api.MechanismDriver):
         self._plugin_property = None
         self._ovn_client_inst = None
         self._maintenance_thread = None
+        self.node_uuid = None
+        self.hash_ring_group = ovn_const.HASH_RING_ML2_GROUP
         self.sg_enabled = ovn_acl.is_sg_enabled()
+        # NOTE(lucasagomes): _clean_hash_ring() must be called before
+        # self.subscribe() to avoid processes racing when adding or
+        # deleting nodes from the Hash Ring during service initialization
+        self._clean_hash_ring()
         self._post_fork_event = threading.Event()
         if cfg.CONF.SECURITYGROUP.firewall_driver:
             LOG.warning('Firewall driver configuration is ignored')
@@ -130,16 +139,6 @@ class OVNMechanismDriver(api.MechanismDriver):
                                                          self._sb_ovn)
         return self._ovn_client_inst
 
-    @property
-    def nb_ovn(self):
-        # NOTE (twilson): This and sb_ovn can be moved to instance variables
-        # once all references to the private versions are changed
-        return self._nb_ovn
-
-    @property
-    def sb_ovn(self):
-        return self._sb_ovn
-
     def _setup_vif_port_bindings(self):
         self.supported_vnic_types = [portbindings.VNIC_NORMAL,
                                      portbindings.VNIC_DIRECT]
@@ -152,14 +151,18 @@ class OVNMechanismDriver(api.MechanismDriver):
                 portbindings.VHOST_USER_MODE:
                 portbindings.VHOST_USER_MODE_SERVER,
                 portbindings.VHOST_USER_OVS_PLUG: True
-            }
+            },
+            portbindings.VIF_DETAILS_CONNECTIVITY:
+                portbindings.CONNECTIVITY_L2,
         }
 
     def subscribe(self):
+        registry.subscribe(self.pre_fork_initialize,
+                           resources.PROCESS,
+                           events.BEFORE_SPAWN)
         registry.subscribe(self.post_fork_initialize,
                            resources.PROCESS,
                            events.AFTER_INIT)
-
         registry.subscribe(self._add_segment_host_mapping_for_segment,
                            resources.SEGMENT,
                            events.AFTER_CREATE)
@@ -188,13 +191,28 @@ class OVNMechanismDriver(api.MechanismDriver):
                                resources.SECURITY_GROUP_RULE,
                                events.BEFORE_DELETE)
 
+    def _clean_hash_ring(self, *args, **kwargs):
+        db_hash_ring.remove_nodes_from_host(self.hash_ring_group)
+
+    def pre_fork_initialize(self, resource, event, trigger, payload=None):
+        """Pre-initialize the ML2/OVN driver."""
+        atexit.register(self._clean_hash_ring)
+        signal.signal(signal.SIGTERM, self._clean_hash_ring)
+
     def post_fork_initialize(self, resource, event, trigger, payload=None):
         # NOTE(rtheis): This will initialize all workers (API, RPC,
         # plugin service and OVN) with OVN IDL connections.
         self._post_fork_event.clear()
         self._ovn_client_inst = None
-        self._nb_ovn, self._sb_ovn = impl_idl_ovn.get_ovn_idls(self,
-                                                               trigger)
+
+        is_maintenance = (utils.get_method_class(trigger) ==
+                          worker.MaintenanceWorker)
+        if not is_maintenance:
+            self.node_uuid = db_hash_ring.add_node(self.hash_ring_group)
+
+        self._nb_ovn, self._sb_ovn = impl_idl_ovn.get_ovn_idls(
+            self, trigger, binding_events=not is_maintenance)
+
         # Override agents API methods
         self.patch_plugin_merge("get_agents", get_agents)
         self.patch_plugin_choose("get_agent", get_agent)
@@ -204,8 +222,8 @@ class OVNMechanismDriver(api.MechanismDriver):
         # Now IDL connections can be safely used.
         self._post_fork_event.set()
 
-        if utils.get_method_class(trigger) == worker.OvnWorker:
-            # Call the synchronization task if its ovn worker
+        if is_maintenance:
+            # Call the synchronization task if its maintenance worker
             # This sync neutron DB to OVN-NB DB only in inconsistent states
             self.nb_synchronizer = ovn_db_sync.OvnNbSynchronizer(
                 self._plugin,
@@ -224,10 +242,12 @@ class OVNMechanismDriver(api.MechanismDriver):
             )
             self.sb_synchronizer.sync()
 
-        if utils.get_method_class(trigger) == maintenance.MaintenanceWorker:
             self._maintenance_thread = maintenance.MaintenanceThread()
             self._maintenance_thread.add_periodics(
                 maintenance.DBInconsistenciesPeriodics(self._ovn_client))
+            self._maintenance_thread.add_periodics(
+                maintenance.HashRingHealthCheckPeriodics(
+                    self.hash_ring_group))
             self._maintenance_thread.start()
 
     def _create_security_group_precommit(self, resource, event, trigger,
@@ -656,8 +676,8 @@ class OVNMechanismDriver(api.MechanismDriver):
         # cannot be found.
         chassis_physnets = []
         try:
-            datapath_type, iface_types, chassis_physnets = \
-                self._sb_ovn.get_chassis_data_for_ml2_bind_port(context.host)
+            datapath_type, iface_types, chassis_physnets = (
+                self._sb_ovn.get_chassis_data_for_ml2_bind_port(context.host))
             iface_types = iface_types.split(',') if iface_types else []
         except RuntimeError:
             LOG.debug('Refusing to bind port %(port_id)s due to '
@@ -689,8 +709,8 @@ class OVNMechanismDriver(api.MechanismDriver):
                          {'port_id': port['id'],
                           'network_type': network_type})
 
-            if (network_type in ['flat', 'vlan']) and \
-               (physical_network not in chassis_physnets):
+            if ((network_type in ['flat', 'vlan']) and
+                    (physical_network not in chassis_physnets)):
                 LOG.info('Refusing to bind port %(port_id)s on '
                          'host %(host)s due to the OVN chassis '
                          'bridge mapping physical networks '
@@ -701,8 +721,8 @@ class OVNMechanismDriver(api.MechanismDriver):
                           'chassis_physnets': chassis_physnets,
                           'physical_network': physical_network})
             else:
-                if datapath_type == ovn_const.CHASSIS_DATAPATH_NETDEV and (
-                    ovn_const.CHASSIS_IFACE_DPDKVHOSTUSER in iface_types):
+                if (datapath_type == ovn_const.CHASSIS_DATAPATH_NETDEV and
+                        ovn_const.CHASSIS_IFACE_DPDKVHOSTUSER in iface_types):
                     vhost_user_socket = utils.ovn_vhu_sockpath(
                         config.get_ovn_vhost_sock_dir(), port['id'])
                     vif_type = portbindings.VIF_TYPE_VHOST_USER
@@ -710,8 +730,8 @@ class OVNMechanismDriver(api.MechanismDriver):
                         portbindings.VHOST_USER_SOCKET: vhost_user_socket
                         })
                     vif_details = dict(self.vif_details[vif_type])
-                    vif_details[portbindings.VHOST_USER_SOCKET] = \
-                        vhost_user_socket
+                    vif_details[portbindings.VHOST_USER_SOCKET] = (
+                        vhost_user_socket)
                 else:
                     vif_type = portbindings.VIF_TYPE_OVS
                     vif_details = self.vif_details[vif_type]
@@ -727,7 +747,22 @@ class OVNMechanismDriver(api.MechanismDriver):
         workers, can return a sequence of worker instances.
         """
         # See doc/source/design/ovn_worker.rst for more details.
-        return [worker.OvnWorker(), maintenance.MaintenanceWorker()]
+        return [worker.MaintenanceWorker()]
+
+    def _update_subport_host_if_needed(self, port_id):
+        parent_port = self._ovn_client.get_parent_port(port_id)
+        if parent_port:
+            admin_context = n_context.get_admin_context()
+            try:
+                port = self._plugin.get_port(admin_context, parent_port)
+                host_id = port.get(portbindings.HOST_ID, '')
+                subport = {
+                    'port': {'binding:host_id': host_id,
+                             'device_owner': trunk_consts.TRUNK_SUBPORT_OWNER}}
+                self._plugin.update_port(admin_context, port_id, subport)
+            except (os_db_exc.DBReferenceError, n_exc.PortNotFound):
+                LOG.debug("Error trying to set host_id %s for subport %s",
+                          host_id, port_id)
 
     def _update_dnat_entry_if_needed(self, port_id, up=True):
         """Update DNAT entry if using distributed floating ips."""
@@ -783,6 +818,11 @@ class OVNMechanismDriver(api.MechanismDriver):
         self._update_dnat_entry_if_needed(port_id)
         self._wait_for_metadata_provisioned_if_needed(port_id)
 
+        # If this port is a subport, we need to update the host_id and set it
+        # to its parent's. Otherwise, Neutron won't even try to bind it and
+        # it will not transition from DOWN to ACTIVE.
+        self._update_subport_host_if_needed(port_id)
+
         provisioning_blocks.provisioning_complete(
             n_context.get_admin_context(),
             port_id,
