@@ -150,8 +150,8 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
         lswitch_names = set([])
         for network in self.core_plugin.get_networks(context):
             lswitch_names.add(network['id'])
-        acl_dict, ignore1, ignore2 = \
-            self.ovn_api.get_acls_for_lswitches(lswitch_names)
+        acl_dict, ignore1, ignore2 = (
+            self.ovn_api.get_acls_for_lswitches(lswitch_names))
         acl_list = list(itertools.chain(*acl_dict.values()))
         acl_list_dict = {}
         for acl in acl_list:
@@ -275,8 +275,8 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
                             neutron_sgs[name]['addresses'].extend(
                                 addresses[ip_version])
 
-            sgnames_to_add, sgnames_to_delete, sgs_to_update =\
-                self.compute_address_set_difference(neutron_sgs, nb_sgs)
+            sgnames_to_add, sgnames_to_delete, sgs_to_update = (
+                self.compute_address_set_difference(neutron_sgs, nb_sgs))
 
         LOG.debug('Address_Sets added %d, removed %d, updated %d',
                   len(sgnames_to_add), len(sgnames_to_delete),
@@ -763,8 +763,8 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
         db_subnets = {}
         filters = {'enable_dhcp': [1]}
         for subnet in self.core_plugin.get_subnets(ctx, filters=filters):
-            if subnet['ip_version'] == constants.IP_VERSION_6 and (
-                subnet.get('ipv6_address_mode') == constants.IPV6_SLAAC):
+            if (subnet['ip_version'] == constants.IP_VERSION_6 and
+                    subnet.get('ipv6_address_mode') == constants.IPV6_SLAAC):
                 continue
             db_subnets[subnet['id']] = subnet
 
@@ -902,8 +902,10 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
                 device_owner=[constants.DEVICE_OWNER_DHCP]))
 
             for port in dhcp_ports:
-                # Do not touch the Neutron DHCP agents ports
-                if utils.is_neutron_dhcp_agent_port(port):
+                # Ports with a device_id like 'dhcp<host_id>-<network_id>'
+                # belong to Neutron DHCP agent so we'll skip those for OVN
+                # metadata.
+                if port['device_id'].startswith('dhcp'):
                     dhcp_ports.remove(port)
 
             if not dhcp_ports:
