@@ -14,10 +14,8 @@ import contextlib
 import uuid
 
 from neutron_lib import exceptions as n_exc
-from oslo_log import log
-import tenacity
-
 from neutron_lib.utils import helpers
+from oslo_log import log
 from oslo_utils import uuidutils
 from ovsdbapp.backend import ovs_idl
 from ovsdbapp.backend.ovs_idl import connection
@@ -26,6 +24,7 @@ from ovsdbapp.backend.ovs_idl import transaction as idl_trans
 from ovsdbapp.backend.ovs_idl import vlog
 from ovsdbapp.schema.ovn_northbound import impl_idl as nb_impl_idl
 from ovsdbapp.schema.ovn_southbound import impl_idl as sb_impl_idl
+import tenacity
 
 from networking_ovn._i18n import _
 from networking_ovn.common import config as cfg
@@ -57,7 +56,6 @@ class OvnNbTransaction(idl_trans.Transaction):
 # and therefor allows networking-ovn to manage connection scope on its own
 class Backend(ovs_idl.Backend):
     lookup_table = {}
-    ovsdb_connection = None
 
     def __init__(self, connection):
         self.ovsdb_connection = connection
@@ -122,7 +120,7 @@ class OvsdbConnectionUnavailable(n_exc.ServiceUnavailable):
 
 # Retry forever to get the OVN NB and SB IDLs. Wait 2^x * 1 seconds between
 # each retry, up to 180 seconds, then 180 seconds afterwards.
-def get_ovn_idls(driver, trigger):
+def get_ovn_idls(driver, trigger, binding_events=False):
     @tenacity.retry(
         wait=tenacity.wait_exponential(max=180),
         reraise=True)
@@ -130,28 +128,29 @@ def get_ovn_idls(driver, trigger):
         trigger_class = utils.get_method_class(trigger)
         LOG.info('Getting %(cls)s for %(trigger)s with retry',
                  {'cls': cls.__name__, 'trigger': trigger_class.__name__})
-        return cls(get_connection(cls, trigger, driver))
+        return cls(get_connection(cls, trigger, driver, binding_events))
 
     vlog.use_python_logger(max_level=cfg.get_ovn_ovsdb_log_level())
     return tuple(get_ovn_idl_retry(c) for c in (OvsdbNbOvnIdl, OvsdbSbOvnIdl))
 
 
-def get_connection(db_class, trigger=None, driver=None):
-    # The trigger is the start() method of the worker class
+def get_connection(db_class, trigger=None, driver=None, binding_events=False):
     if db_class == OvsdbNbOvnIdl:
         args = (cfg.get_ovn_nb_connection(), 'OVN_Northbound')
-        cls = ovsdb_monitor.OvnNbIdl
     elif db_class == OvsdbSbOvnIdl:
         args = (cfg.get_ovn_sb_connection(), 'OVN_Southbound')
-        cls = ovsdb_monitor.OvnSbIdl
 
-    if trigger and utils.get_method_class(trigger).__name__ == 'OvnWorker':
-        idl_ = cls.from_server(*args, driver=driver)
-    else:
-        if db_class == OvsdbSbOvnIdl:
-            idl_ = ovsdb_monitor.BaseOvnSbIdl.from_server(*args)
+    if binding_events:
+        if db_class == OvsdbNbOvnIdl:
+            idl_ = ovsdb_monitor.OvnNbIdl.from_server(*args, driver=driver)
         else:
+            idl_ = ovsdb_monitor.OvnSbIdl.from_server(*args, driver=driver)
+    else:
+        if db_class == OvsdbNbOvnIdl:
             idl_ = ovsdb_monitor.BaseOvnIdl.from_server(*args)
+        else:
+            idl_ = ovsdb_monitor.BaseOvnSbIdl.from_server(*args)
+
     return connection.Connection(idl_, timeout=cfg.get_ovn_ovsdb_timeout())
 
 
@@ -437,46 +436,22 @@ class OvsdbNbOvnIdl(nb_impl_idl.OvnNbApiIdlImpl, Backend):
         except idlutils.RowNotFound:
             return []
 
-    def get_chassis_gateways(self, chassis_name):
-        gw_chassis = self.db_find_rows(
-            'Gateway_Chassis', ('chassis_name', '=', chassis_name))
-        return gw_chassis.execute(check_error=True)
-
-    def get_unhosted_gateways(self, port_physnet_dict, chassis_with_physnets,
-                              all_gw_chassis):
-        unhosted_gateways = set()
-        for port, physnet in port_physnet_dict.items():
-            lrp_name = '%s%s' % (ovn_const.LRP_PREFIX, port)
-            original_state = self.get_gateway_chassis_binding(lrp_name)
-
-            # Filter out chassis that lost physnet, the cms option,
-            # or has been deleted.
-            actual_gw_chassis = [
-                chassis for chassis in original_state
-                if not utils.is_gateway_chassis_invalid(
-                    chassis, all_gw_chassis, physnet, chassis_with_physnets)]
-
-            # Check if gw ports are fully scheduled.
-            if len(actual_gw_chassis) >= ovn_const.MAX_GW_CHASSIS:
+    def get_unhosted_gateways(self, port_physnet_dict, chassis_physnets,
+                              gw_chassis):
+        unhosted_gateways = []
+        for lrp in self._tables['Logical_Router_Port'].rows.values():
+            if not lrp.name.startswith('lrp-'):
                 continue
-
-            # If there are no gateways with 'enable-chassis-as-gw' cms option
-            # then try to schedule on all gateways with physnets connected,
-            # and filter required physnet.
-            available_chassis = {
-                c for c in all_gw_chassis or chassis_with_physnets.keys()
-                if not utils.is_gateway_chassis_invalid(
-                    c, all_gw_chassis, physnet, chassis_with_physnets)}
-
-            if available_chassis == set(original_state):
-                # The same situation as was before. Nothing
-                # to be rescheduled.
-                continue
-            if not available_chassis:
-                # There is no chassis that could host
-                # this gateway.
-                continue
-            unhosted_gateways.add(lrp_name)
+            physnet = port_physnet_dict.get(lrp.name[len('lrp-'):])
+            chassis_list = self._get_logical_router_port_gateway_chassis(lrp)
+            is_max_gw_reached = len(chassis_list) < ovn_const.MAX_GW_CHASSIS
+            for chassis_name, prio in chassis_list:
+                # TODO(azbiswas): Handle the case when a chassis is no
+                # longer valid. This may involve moving conntrack states,
+                # so it needs to discussed in the OVN community first.
+                if is_max_gw_reached or utils.is_gateway_chassis_invalid(
+                        chassis_name, gw_chassis, physnet, chassis_physnets):
+                    unhosted_gateways.append(lrp.name)
         return unhosted_gateways
 
     def add_dhcp_options(self, subnet_id, port_id=None, may_exist=True,
@@ -699,50 +674,27 @@ class OvsdbNbOvnIdl(nb_impl_idl.OvnNbApiIdlImpl, Backend):
     def get_port_group(self, pg_name):
         if uuidutils.is_uuid_like(pg_name):
             pg_name = utils.ovn_port_group_name(pg_name)
-        try:
-            for pg in self._tables['Port_Group'].rows.values():
-                if pg.name == pg_name:
-                    return pg
-        except KeyError:
-            # TODO(dalvarez): This except block is added for backwards compat
-            # with old OVN schemas (<=2.9) where Port Groups are not present.
-            # This (and other conditional code around this feature) shall be
-            # removed at some point.
-            return
+
+        for pg in self._tables['Port_Group'].rows.values():
+            if pg.name == pg_name:
+                return pg
 
     def get_port_groups(self):
         port_groups = {}
-        try:
-            for row in self._tables['Port_Group'].rows.values():
-                name = getattr(row, 'name')
-                if not (ovn_const.OVN_SG_EXT_ID_KEY in row.external_ids or
-                   name == ovn_const.OVN_DROP_PORT_GROUP_NAME):
-                    continue
-                data = {}
-                for row_key in getattr(row, "_data", {}):
-                    data[row_key] = getattr(row, row_key)
-                port_groups[name] = data
-        except KeyError:
-            # TODO(dalvarez): This except block is added for backwards compat
-            # with old OVN schemas (<=2.9) where Port Groups are not present.
-            # This (and other conditional code around this feature) shall be
-            # removed at some point.
-            pass
+        for row in self._tables['Port_Group'].rows.values():
+            name = getattr(row, 'name')
+            if not (ovn_const.OVN_SG_EXT_ID_KEY in row.external_ids or
+               name == ovn_const.OVN_DROP_PORT_GROUP_NAME):
+                continue
+            data = {}
+            for row_key in getattr(row, "_data", {}):
+                data[row_key] = getattr(row, row_key)
+            port_groups[name] = data
         return port_groups
 
     def check_liveness(self):
         return cmd.CheckLivenessCommand(self)
 
-    def set_lswitch_port_to_virtual_type(self, lport_name, vip,
-                                         virtual_parent, if_exists=True):
-        return cmd.SetLSwitchPortToVirtualTypeCommand(
-            self, lport_name, vip, virtual_parent, if_exists)
-
-    def unset_lswitch_port_to_virtual_type(self, lport_name,
-                                           virtual_parent, if_exists=True):
-        return cmd.UnsetLSwitchPortToVirtualTypeCommand(
-            self, lport_name, virtual_parent, if_exists)
-
 
 class OvsdbSbOvnIdl(sb_impl_idl.OvnSbApiIdlImpl, Backend):
     def __init__(self, connection):
