@@ -13,11 +13,14 @@
 #    under the License.
 
 import abc
+import datetime
 
 from neutron_lib.plugins import constants
 from neutron_lib.plugins import directory
 from neutron_lib.utils import helpers
+from oslo_config import cfg
 from oslo_log import log
+from oslo_utils import timeutils
 from ovs.stream import Stream
 from ovsdbapp.backend.ovs_idl import connection
 from ovsdbapp.backend.ovs_idl import event as row_event
@@ -27,8 +30,12 @@ from ovsdbapp import event
 from networking_ovn.agent import stats
 from networking_ovn.common import config as ovn_config
 from networking_ovn.common import constants as ovn_const
+from networking_ovn.common import exceptions
+from networking_ovn.common import hash_ring_manager
 from networking_ovn.common import utils
+from networking_ovn.db import hash_ring as db_hash_ring
 
+CONF = cfg.CONF
 LOG = log.getLogger(__name__)
 
 
@@ -116,22 +123,7 @@ class ChassisEvent(row_event.RowEvent):
         super(ChassisEvent, self).__init__(events, table, None)
         self.event_name = 'ChassisEvent'
 
-    def match_fn(self, event, row, old):
-        if event != self.ROW_UPDATE:
-            return True
-        # NOTE(lucasgomes): If the external_ids column wasn't updated
-        # (meaning, Chassis "gateway" status didn't change) just returns
-        if not hasattr(old, 'external_ids'):
-            return
-        if (old.external_ids.get('ovn-bridge-mappings') !=
-                row.external_ids.get('ovn-bridge-mappings')):
-            return True
-        f = utils.is_gateway_chassis
-        return f(old) != f(row)
-
     def run(self, event, row, old):
-        if not self.match_fn(event, row, old):
-            return
         host = row.hostname
         phy_nets = []
         if event != self.ROW_DELETE:
@@ -141,36 +133,11 @@ class ChassisEvent(row_event.RowEvent):
             phy_nets = list(mapping_dict)
 
         self.driver.update_segment_host_mapping(host, phy_nets)
-
         if utils.is_ovn_l3(self.l3_plugin):
-            # If chassis lost physnet or has been
-            # deleted we can limit the scope and
-            # reschedule only ports from this chassis.
-            # In other cases we need to reschedule all gw ports.
-            kwargs = {'event_from_chassis': None}
-            if event == self.ROW_DELETE:
-                kwargs['event_from_chassis'] = row.name
-            elif event == self.ROW_UPDATE:
-                old_mappings = old.external_ids.get('ovn-bridge-mappings',
-                                                    set()) or set()
-                new_mappings = row.external_ids.get('ovn-bridge-mappings',
-                                                    set()) or set()
-                if old_mappings:
-                    old_mappings = set(old_mappings.split(','))
-                if new_mappings:
-                    new_mappings = set(new_mappings.split(','))
-
-                mappings_removed = old_mappings - new_mappings
-                mappings_added = new_mappings - old_mappings
-                if mappings_removed and not mappings_added:
-                    # Mapping has been only removed. So we can
-                    # limit scope of rescheduling only to impacted
-                    # gateway chassis.
-                    kwargs['event_from_chassis'] = row.name
-            self.l3_plugin.schedule_unhosted_gateways(**kwargs)
+            self.l3_plugin.schedule_unhosted_gateways()
 
 
-class PortBindingChassisUpdateEvent(BaseEvent):
+class PortBindingChassisUpdateEvent(row_event.RowEvent):
     """Event for matching a port moving chassis
 
     If the LSP is up and the Port_Binding chassis has just changed,
@@ -180,12 +147,13 @@ class PortBindingChassisUpdateEvent(BaseEvent):
     only monitors for transitions from DOWN to UP.
     """
 
-    table = 'Port_Binding'
-    events = (BaseEvent.ROW_UPDATE,)
-
     def __init__(self, driver):
         self.driver = driver
-        super(PortBindingChassisUpdateEvent, self).__init__()
+        table = 'Port_Binding'
+        events = (self.ROW_UPDATE,)
+        super(PortBindingChassisUpdateEvent, self).__init__(
+            events, table, None)
+        self.event_name = self.__class__.__name__
 
     def match_fn(self, event, row, old=None):
         # NOTE(twilson) ROW_UPDATE events always pass old, but chassis will
@@ -418,7 +386,56 @@ class OvnIdl(BaseOvnIdl):
         """Should be called after the idl has been initialized"""
 
 
-class OvnNbIdl(OvnIdl):
+class OvnIdlDistributedLock(BaseOvnIdl):
+
+    def __init__(self, driver, remote, schema):
+        super(OvnIdlDistributedLock, self).__init__(remote, schema)
+        self.driver = driver
+        self.notify_handler = OvnDbNotifyHandler(driver)
+        self._node_uuid = self.driver.node_uuid
+        self._hash_ring = hash_ring_manager.HashRingManager(
+            self.driver.hash_ring_group)
+        self._last_touch = None
+
+    def notify(self, event, row, updates=None):
+        try:
+            target_node = self._hash_ring.get_node(str(row.uuid))
+        except exceptions.HashRingIsEmpty as e:
+            LOG.error('HashRing is empty, error: %s', e)
+            return
+
+        if target_node != self._node_uuid:
+            return
+
+        # If the worker hasn't been health checked by the maintenance
+        # thread (see bug #1834498), indicate that it's alive here
+        time_now = timeutils.utcnow()
+        touch_timeout = time_now - datetime.timedelta(
+            seconds=ovn_const.HASH_RING_TOUCH_INTERVAL)
+        if not self._last_touch or touch_timeout >= self._last_touch:
+            # NOTE(lucasagomes): Guard the db operation with an exception
+            # handler. If heartbeating fails for whatever reason, log
+            # the error and continue with processing the event
+            try:
+                db_hash_ring.touch_node(self._node_uuid)
+                self._last_touch = time_now
+            except Exception as e:
+                LOG.exception('Hash Ring node %s failed to heartbeat',
+                              self._node_uuid)
+
+        LOG.debug('Hash Ring: Node %(node)s (host: %(hostname)s) '
+                  'handling event "%(event)s" for row %(row)s '
+                  '(table: %(table)s)',
+                  {'node': self._node_uuid, 'hostname': CONF.host,
+                   'event': event, 'row': row.uuid, 'table': row._table.name})
+        self.notify_handler.notify(event, row, updates)
+
+    @abc.abstractmethod
+    def post_connect(self):
+        """Should be called after the idl has been initialized"""
+
+
+class OvnNbIdl(OvnIdlDistributedLock):
 
     def __init__(self, driver, remote, schema):
         super(OvnNbIdl, self).__init__(driver, remote, schema)
@@ -440,9 +457,7 @@ class OvnNbIdl(OvnIdl):
         _check_and_set_ssl_files(schema_name)
         helper = idlutils.get_schema_helper(connection_string, schema_name)
         helper.register_all()
-        _idl = cls(driver, connection_string, helper)
-        _idl.set_lock(_idl.event_lock_name)
-        return _idl
+        return cls(driver, connection_string, helper)
 
     def unwatch_logical_switch_port_create_events(self):
         """Unwatch the logical switch port create events.
@@ -462,7 +477,7 @@ class OvnNbIdl(OvnIdl):
         self.unwatch_logical_switch_port_create_events()
 
 
-class OvnSbIdl(OvnIdl):
+class OvnSbIdl(OvnIdlDistributedLock):
 
     @classmethod
     def from_server(cls, connection_string, schema_name, driver):
@@ -473,9 +488,7 @@ class OvnSbIdl(OvnIdl):
         helper.register_table('Port_Binding')
         helper.register_table('Datapath_Binding')
         helper.register_table('MAC_Binding')
-        _idl = cls(driver, connection_string, helper)
-        _idl.set_lock(_idl.event_lock_name)
-        return _idl
+        return cls(driver, connection_string, helper)
 
     def post_connect(self):
         """Watch Chassis events.
