@@ -12,7 +12,9 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from datetime import datetime
 import os
+import shutil
 import time
 
 import fixtures
@@ -20,7 +22,7 @@ import mock
 from neutron.conf.plugins.ml2 import config
 from neutron.plugins.ml2.drivers import type_geneve  # noqa
 from neutron.tests.unit.plugins.ml2 import test_plugin
-from neutron.tests.unit import testlib_api
+from neutron_lib import fixture
 from neutron_lib.plugins import constants
 from neutron_lib.plugins import directory
 from oslo_concurrency import lockutils
@@ -29,7 +31,6 @@ from oslo_db import exception as os_db_exc
 from oslo_db.sqlalchemy import provision
 from oslo_log import log
 from oslo_utils import uuidutils
-from ovsdbapp.backend.ovs_idl import command
 from ovsdbapp.backend.ovs_idl import connection
 
 # Load all the models to register them into SQLAlchemy metadata before using
@@ -49,27 +50,6 @@ DEFAULT_LOG_DIR = os.path.join(os.environ.get('OS_LOG_PATH', '/tmp'),
 SQL_FIXTURE_LOCK = 'sql_fixture_lock'
 
 
-class AddFakeChassisCommand(command.BaseCommand):
-    """Add a fake chassis in OVN SB DB for functional test."""
-
-    def __init__(self, api, name, ip, **columns):
-        super(AddFakeChassisCommand, self).__init__(api)
-        self.name = name
-        self.ip = ip
-        self.columns = columns
-
-    def run_idl(self, txn):
-        encap_row = txn.insert(self.api._tables['Encap'])
-        encap_row.type = 'geneve'
-        encap_row.ip = self.ip
-        self.columns.update({'encaps': [encap_row.uuid]})
-
-        row = txn.insert(self.api._tables['Chassis'])
-        row.name = self.name
-        for col, val in self.columns.items():
-            setattr(row, col, val)
-
-
 class ConnectionFixture(fixtures.Fixture):
     def __init__(self, idl=None, constr=None, schema=None, timeout=60):
         self.idl = idl or ovsdb_monitor.BaseOvnIdl.from_server(
@@ -85,7 +65,7 @@ class ConnectionFixture(fixtures.Fixture):
         self.connection.stop()
 
 
-class OVNSqlFixture(testlib_api.StaticSqlFixture):
+class OVNSqlFixture(fixture.StaticSqlFixture):
 
     @classmethod
     @lockutils.synchronized(SQL_FIXTURE_LOCK)
@@ -117,7 +97,7 @@ class TestOVNFunctionalBase(test_plugin.Ml2PluginV2TestCase):
     _counter = 0
     l3_plugin = 'networking_ovn.l3.l3_ovn.OVNL3RouterPlugin'
 
-    def setUp(self, ovn_worker=False):
+    def setUp(self, maintenance_worker=False):
         config.cfg.CONF.set_override('extension_drivers',
                                      self._extension_drivers,
                                      group='ml2')
@@ -132,15 +112,17 @@ class TestOVNFunctionalBase(test_plugin.Ml2PluginV2TestCase):
                                      group='ovn')
 
         super(TestOVNFunctionalBase, self).setUp()
+        self.test_log_dir = os.path.join(DEFAULT_LOG_DIR, self.id())
         base.setup_test_logging(
-            cfg.CONF, DEFAULT_LOG_DIR, "%s.txt" % self.id())
+            cfg.CONF, self.test_log_dir, "testrun.txt")
 
         mm = directory.get_plugin().mechanism_manager
         self.mech_driver = mm.mech_drivers['ovn'].obj
         self.l3_plugin = directory.get_plugin(constants.L3)
         self.ovsdb_server_mgr = None
         self.ovn_northd_mgr = None
-        self.ovn_worker = ovn_worker
+        self.maintenance_worker = maintenance_worker
+        self.temp_dir = self.useFixture(fixtures.TempDir()).path
         self._start_ovsdb_server_and_idls()
         self._start_ovn_northd()
 
@@ -177,7 +159,6 @@ class TestOVNFunctionalBase(test_plugin.Ml2PluginV2TestCase):
                               protocol=self._ovsdb_protocol))
 
     def _start_ovsdb_server_and_idls(self):
-        self.temp_dir = self.useFixture(fixtures.TempDir()).path
         # Start 2 ovsdb-servers one each for OVN NB DB and OVN SB DB
         # ovsdb-server with OVN SB DB can be used to test the chassis up/down
         # events.
@@ -247,18 +228,36 @@ class TestOVNFunctionalBase(test_plugin.Ml2PluginV2TestCase):
                 pass
 
         trigger_cls = TriggerCls()
-        if self.ovn_worker:
-            trigger_cls.trigger.__self__.__class__ = worker.OvnWorker
+        if self.maintenance_worker:
+            trigger_cls.trigger.__self__.__class__ = worker.MaintenanceWorker
             cfg.CONF.set_override('neutron_sync_mode', 'off', 'ovn')
 
+        self.addCleanup(self._collect_processes_logs)
         self.addCleanup(self.stop)
 
         # mech_driver.post_fork_initialize creates the IDL connections
         self.mech_driver.post_fork_initialize(
             mock.ANY, mock.ANY, trigger_cls.trigger)
 
+    def _collect_processes_logs(self):
+        for database in ("nb", "sb"):
+            for file_suffix in ("log", "db"):
+                src_filename = "ovn_%(db)s.%(suffix)s" % {
+                    'db': database,
+                    'suffix': file_suffix
+                }
+                dst_filename = "ovn_%(db)s-%(timestamp)s.%(suffix)s" % {
+                    'db': database,
+                    'suffix': file_suffix,
+                    'timestamp': datetime.now().strftime('%y-%m-%d_%H-%M-%S'),
+                }
+
+                filepath = os.path.join(self.temp_dir, src_filename)
+                shutil.copyfile(
+                    filepath, os.path.join(self.test_log_dir, dst_filename))
+
     def stop(self):
-        if self.ovn_worker:
+        if self.maintenance_worker:
             self.mech_driver.nb_synchronizer.stop()
             self.mech_driver.sb_synchronizer.stop()
         self.mech_driver._nb_ovn.ovsdb_connection.stop()
@@ -266,12 +265,26 @@ class TestOVNFunctionalBase(test_plugin.Ml2PluginV2TestCase):
 
     def restart(self):
         self.stop()
+        # The OVN sync test starts its own synchronizers...
+        self.l3_plugin._nb_ovn_idl.ovsdb_connection.stop()
+        self.l3_plugin._sb_ovn_idl.ovsdb_connection.stop()
+        # Stop our monitor connections
+        self.nb_api.ovsdb_connection.stop()
+        self.sb_api.ovsdb_connection.stop()
 
         if self.ovsdb_server_mgr:
             self.ovsdb_server_mgr.stop()
         if self.ovn_northd_mgr:
             self.ovn_northd_mgr.stop()
 
+        self.mech_driver._nb_ovn = None
+        self.mech_driver._sb_ovn = None
+        self.l3_plugin._nb_ovn_idl = None
+        self.l3_plugin._sb_ovn_idl = None
+        self.nb_api.ovsdb_connection = None
+        self.sb_api.ovsdb_connection = None
+
+        self.ovsdb_server_mgr.delete_dbs()
         self._start_ovsdb_server_and_idls()
         self._start_ovn_northd()
 
