@@ -15,19 +15,22 @@
 
 import mock
 
+from futurist import periodics
 from neutron.tests.unit.plugins.ml2 import test_security_group as test_sg
 from neutron_lib.db import api as db_api
 
+from networking_ovn.common import config as ovn_config
 from networking_ovn.common import constants
 from networking_ovn.common import maintenance
 from networking_ovn.common import utils
 from networking_ovn.db import maintenance as db_maint
 from networking_ovn.db import revision as db_rev
+from networking_ovn import ovn_db_sync
 from networking_ovn.tests.unit.db import base as db_base
 
 
 @mock.patch.object(maintenance.DBInconsistenciesPeriodics,
-                   'has_lock', lambda _: True)
+                   'has_lock', mock.PropertyMock(return_value=True))
 class TestDBInconsistenciesPeriodics(db_base.DBTestCase,
                                      test_sg.Ml2SecurityGroupsTestCase):
 
@@ -51,6 +54,56 @@ class TestDBInconsistenciesPeriodics(db_base.DBTestCase,
         self.periodic.check_for_inconsistencies()
         mock_fix_net.assert_called_once_with(fake_row)
 
+    def _test_migrate_to_port_groups_helper(self, pg_supported, a_sets,
+                                            migration_expected, never_again):
+        self.fake_ovn_client._nb_idl.is_port_groups_supported.return_value = (
+            pg_supported)
+        self.fake_ovn_client._nb_idl.get_address_sets.return_value = a_sets
+        with mock.patch.object(ovn_db_sync.OvnNbSynchronizer,
+                               'migrate_to_port_groups') as mtpg:
+            if never_again:
+                self.assertRaises(periodics.NeverAgain,
+                                  self.periodic.migrate_to_port_groups)
+            else:
+                self.periodic.migrate_to_port_groups()
+
+            if migration_expected:
+                mtpg.assert_called_once()
+            else:
+                mtpg.assert_not_called()
+
+    def test_migrate_to_port_groups_port_groups_not_supported(self):
+        self._test_migrate_to_port_groups_helper(pg_supported=False,
+                                                 a_sets=None,
+                                                 migration_expected=False,
+                                                 never_again=True)
+
+    def test_migrate_to_port_groups_not_needed(self):
+        self._test_migrate_to_port_groups_helper(pg_supported=True,
+                                                 a_sets=None,
+                                                 migration_expected=False,
+                                                 never_again=True)
+
+    def test_migrate_to_port_groups(self):
+        # Check normal migration path: if port groups are supported by the
+        # schema and the migration has to be done, it will take place and
+        # won't be attempted in the future.
+        self._test_migrate_to_port_groups_helper(pg_supported=True,
+                                                 a_sets=['as1', 'as2'],
+                                                 migration_expected=True,
+                                                 never_again=True)
+
+    def test_migrate_to_port_groups_no_lock(self):
+        with mock.patch.object(maintenance.DBInconsistenciesPeriodics,
+                               'has_lock', mock.PropertyMock(
+                                   return_value=False)):
+            # Check that if this worker doesn't have the lock, it won't
+            # perform the migration and it will try again later.
+            self._test_migrate_to_port_groups_helper(pg_supported=True,
+                                                     a_sets=['as1', 'as2'],
+                                                     migration_expected=False,
+                                                     never_again=False)
+
     def _test_fix_create_update_network(self, ovn_rev, neutron_rev):
         self.net['revision_number'] = neutron_rev
 
@@ -177,3 +230,41 @@ class TestDBInconsistenciesPeriodics(db_base.DBTestCase,
         l3_mock.add_router_interface.assert_called_once_with(
             mock.ANY, port['device_id'], {'port_id': port['id']},
             may_exist=True)
+
+    @mock.patch.object(maintenance.LOG, 'debug')
+    def test__log_maintenance_inconsistencies(self, mock_log):
+        ovn_config.cfg.CONF.set_override('debug', True)
+
+        # Create fake inconsistencies: 2 networks, 4 subnets and 8 ports
+        incst = []
+        incst += [mock.Mock(resource_type=constants.TYPE_NETWORKS)] * 2
+        incst += [mock.Mock(resource_type=constants.TYPE_SUBNETS)] * 4
+        incst += [mock.Mock(resource_type=constants.TYPE_PORTS)] * 8
+
+        # Create fake inconsistencies for delete: 3 routers and 6 router ports
+        incst_del = []
+        incst_del += [mock.Mock(resource_type=constants.TYPE_ROUTERS)] * 3
+        incst_del += [mock.Mock(resource_type=constants.TYPE_ROUTER_PORTS)] * 6
+
+        self.periodic._log_maintenance_inconsistencies(incst, incst_del)
+
+        # Assert LOG.debug was called twice
+        self.assertEqual(2, len(mock_log.call_args_list))
+
+        # Assert the log matches the number of inconsistencies
+        fail_str_create_update = mock_log.call_args_list[0][0][1]['fail_str']
+        self.assertIn('networks=2', fail_str_create_update)
+        self.assertIn('subnets=4', fail_str_create_update)
+        self.assertIn('ports=8', fail_str_create_update)
+
+        fail_str_delete = mock_log.call_args_list[1][0][1]['fail_str']
+        self.assertIn('routers=3', fail_str_delete)
+        self.assertIn('router_ports=6', fail_str_delete)
+
+    @mock.patch.object(maintenance.LOG, 'debug')
+    def test__log_maintenance_inconsistencies_debug_disabled(self, mock_log):
+        ovn_config.cfg.CONF.set_override('debug', False)
+
+        incst = [mock.Mock(resource_type=constants.TYPE_NETWORKS)] * 2
+        self.periodic._log_maintenance_inconsistencies(incst, [])
+        self.assertFalse(mock_log.called)
