@@ -11,12 +11,12 @@
 #    under the License.
 
 import mock
-from neutron_lib import constants
-from oslo_utils import uuidutils
 
 from neutron.objects.qos import policy as qos_policy
 from neutron.objects.qos import rule as qos_rule
 from neutron.tests import base
+from neutron_lib import constants
+from oslo_utils import uuidutils
 
 from networking_ovn.common import utils
 from networking_ovn.ml2 import qos_driver
@@ -105,7 +105,8 @@ class TestOVNQosDriver(base.BaseTestCase):
                                return_value=return_val) as get_rules:
             options = self.driver._generate_port_options(context, policy_id)
             if policy_id:
-                get_rules.assert_called_once_with(context, policy_id)
+                get_rules.assert_called_once_with(qos_policy.QosPolicy,
+                                                  context, policy_id)
             else:
                 get_rules.assert_not_called()
             self.assertEqual(expected_result, options)
