@@ -13,11 +13,13 @@
 #    under the License.
 
 import copy
+import datetime
 import os
 
 import mock
 from neutron_lib.plugins import constants
 from neutron_lib.plugins import directory
+from oslo_utils import timeutils
 from oslo_utils import uuidutils
 from ovs.db import idl as ovs_idl
 from ovs import poller
@@ -27,7 +29,9 @@ from ovsdbapp.backend.ovs_idl import idlutils
 
 from networking_ovn.common import config as ovn_config
 from networking_ovn.common import constants as ovn_const
+from networking_ovn.common import hash_ring_manager
 from networking_ovn.common import utils
+from networking_ovn.db import hash_ring as db_hash_ring
 from networking_ovn.ovsdb import ovsdb_monitor
 from networking_ovn.tests import base
 from networking_ovn.tests.unit import fakes
@@ -93,8 +97,6 @@ class TestOvnNbIdlNotifyHandler(test_mech_driver.OVNMechanismDriverTestCase):
         helper = ovs_idl.SchemaHelper(schema_json=OVN_NB_SCHEMA)
         helper.register_all()
         self.idl = ovsdb_monitor.OvnNbIdl(self.driver, "remote", helper)
-        self.idl.lock_name = self.idl.event_lock_name
-        self.idl.has_lock = True
         self.lp_table = self.idl.tables.get('Logical_Switch_Port')
         self.driver.set_port_status_up = mock.Mock()
         self.driver.set_port_status_down = mock.Mock()
@@ -198,18 +200,17 @@ class TestOvnNbIdlNotifyHandler(test_mech_driver.OVNMechanismDriverTestCase):
         self.assertFalse(self.driver.set_port_status_up.called)
         self.assertFalse(self.driver.set_port_status_down.called)
 
-    def test_notify_no_ovsdb_lock(self):
-        self.idl.is_lock_contended = True
+    @mock.patch('networking_ovn.common.hash_ring_manager.'
+                'HashRingManager.get_node')
+    def test_notify_different_target_node(self, mock_get_node):
+        mock_get_node.return_value = 'this-is-a-different-node'
+        row = fakes.FakeOvsdbRow.create_one_ovsdb_row()
         self.idl.notify_handler.notify = mock.Mock()
-        self.idl.notify("create", mock.ANY)
+        self.idl.notify("create", row)
+        # Assert that if the target_node returned by the ring is different
+        # than this driver's node_uuid, notify() won't be called
         self.assertFalse(self.idl.notify_handler.notify.called)
 
-    def test_notify_ovsdb_lock_not_yet_contended(self):
-        self.idl.is_lock_contended = False
-        self.idl.notify_handler.notify = mock.Mock()
-        self.idl.notify("create", mock.ANY)
-        self.assertTrue(self.idl.notify_handler.notify.called)
-
 
 class TestOvnSbIdlNotifyHandler(test_mech_driver.OVNMechanismDriverTestCase):
 
@@ -220,8 +221,6 @@ class TestOvnSbIdlNotifyHandler(test_mech_driver.OVNMechanismDriverTestCase):
         sb_helper = ovs_idl.SchemaHelper(schema_json=OVN_SB_SCHEMA)
         sb_helper.register_table('Chassis')
         self.sb_idl = ovsdb_monitor.OvnSbIdl(self.driver, "remote", sb_helper)
-        self.sb_idl.lock_name = self.sb_idl.event_lock_name
-        self.sb_idl.has_lock = True
         self.sb_idl.post_connect()
         self.chassis_table = self.sb_idl.tables.get('Chassis')
         self.driver.update_segment_host_mapping = mock.Mock()
@@ -254,15 +253,17 @@ class TestOvnSbIdlNotifyHandler(test_mech_driver.OVNMechanismDriverTestCase):
         self._test_chassis_helper('create', self.row_json)
         self.driver.update_segment_host_mapping.assert_called_once_with(
             'fake-hostname', ['fake-phynet1'])
-        self.l3_plugin.schedule_unhosted_gateways.assert_called_once_with(
-            event_from_chassis=None)
+        self.assertEqual(
+            1,
+            self.l3_plugin.schedule_unhosted_gateways.call_count)
 
     def test_chassis_delete_event(self):
         self._test_chassis_helper('delete', self.row_json)
         self.driver.update_segment_host_mapping.assert_called_once_with(
             'fake-hostname', [])
-        self.l3_plugin.schedule_unhosted_gateways.assert_called_once_with(
-            event_from_chassis='fake-name')
+        self.assertEqual(
+            1,
+            self.l3_plugin.schedule_unhosted_gateways.call_count)
 
     def test_chassis_update_event(self):
         old_row_json = copy.deepcopy(self.row_json)
@@ -271,53 +272,9 @@ class TestOvnSbIdlNotifyHandler(test_mech_driver.OVNMechanismDriverTestCase):
         self._test_chassis_helper('update', self.row_json, old_row_json)
         self.driver.update_segment_host_mapping.assert_called_once_with(
             'fake-hostname', ['fake-phynet1'])
-        self.l3_plugin.schedule_unhosted_gateways.assert_called_once_with(
-            event_from_chassis=None)
-
-    def test_chassis_update_event_reschedule_not_needed(self):
-        self.row_json['external_ids'][1].append(['foo_field', 'foo_value_new'])
-        old_row_json = copy.deepcopy(self.row_json)
-        old_row_json['external_ids'][1][1][1] = (
-            "foo_value")
-        self._test_chassis_helper('update', self.row_json, old_row_json)
-        self.driver.update_segment_host_mapping.assert_not_called()
-        self.l3_plugin.schedule_unhosted_gateways.assert_not_called()
-
-    def test_chassis_update_event_reschedule_lost_physnet(self):
-        old_row_json = copy.deepcopy(self.row_json)
-        self.row_json['external_ids'][1][0][1] = ''
-        self._test_chassis_helper('update', self.row_json, old_row_json)
-        self.l3_plugin.schedule_unhosted_gateways.assert_called_once_with(
-            event_from_chassis='fake-name')
-
-    def test_chassis_update_event_reschedule_add_physnet(self):
-        old_row_json = copy.deepcopy(self.row_json)
-        self.row_json['external_ids'][1][0][1] += ',foo_physnet:foo_br'
-        self._test_chassis_helper('update', self.row_json, old_row_json)
-        self.assertEqual(self.driver.update_segment_host_mapping.call_count, 1)
-        calls = self.driver.update_segment_host_mapping.mock_calls
-        self.assertEqual(calls[0][1][0], 'fake-hostname')
-        self.assertListEqual(
-            sorted(calls[0][1][1]),
-            sorted(['fake-phynet1', 'foo_physnet']))
-        self.l3_plugin.schedule_unhosted_gateways.assert_called_once_with(
-            event_from_chassis=None)
-
-    def test_chassis_update_event_reschedule_add_and_remove_physnet(self):
-        old_row_json = copy.deepcopy(self.row_json)
-        self.row_json['external_ids'][1][0][1] = 'foo_physnet:foo_br'
-        self._test_chassis_helper('update', self.row_json, old_row_json)
-        self.driver.update_segment_host_mapping.assert_called_once_with(
-            'fake-hostname', ['foo_physnet'])
-        self.l3_plugin.schedule_unhosted_gateways.assert_called_once_with(
-            event_from_chassis=None)
-
-    def test_chassis_update_empty_no_external_ids(self):
-        old_row_json = copy.deepcopy(self.row_json)
-        old_row_json.pop('external_ids')
-        self._test_chassis_helper('update', self.row_json, old_row_json)
-        self.driver.update_segment_host_mapping.assert_not_called()
-        self.l3_plugin.schedule_unhosted_gateways.assert_not_called()
+        self.assertEqual(
+            1,
+            self.l3_plugin.schedule_unhosted_gateways.call_count)
 
 
 class TestOvnDbNotifyHandler(base.TestCase):
@@ -392,7 +349,7 @@ class TestOvnConnection(base.TestCase):
         _idl = idl_class.from_server('punix:/tmp/fake', schema, mock.Mock())
         self.ovn_connection = connection.Connection(_idl, mock.Mock())
         with mock.patch.object(poller, 'Poller'), \
-            mock.patch('threading.Thread'):
+                mock.patch('threading.Thread'):
             self.ovn_connection.start()
             # A second start attempt shouldn't re-register.
             self.ovn_connection.start()
@@ -473,6 +430,84 @@ class TestChassisMetadataAgentEvent(base.TestCase):
         self.assertTrue(self.event.match_fn(ROW_UPDATE, row, old=old))
 
 
+class TestOvnIdlDistributedLock(base.TestCase):
+
+    def setUp(self):
+        super(TestOvnIdlDistributedLock, self).setUp()
+        self.node_uuid = uuidutils.generate_uuid()
+        self.fake_driver = mock.Mock()
+        self.fake_driver.node_uuid = self.node_uuid
+        self.fake_event = 'fake-event'
+        self.fake_row = fakes.FakeOvsdbRow.create_one_ovsdb_row(
+            attrs={'_table': mock.Mock(name='FakeTable')})
+        helper = ovs_idl.SchemaHelper(schema_json=OVN_NB_SCHEMA)
+        helper.register_all()
+
+        with mock.patch.object(ovsdb_monitor, 'OvnDbNotifyHandler'):
+            self.idl = ovsdb_monitor.OvnIdlDistributedLock(
+                self.fake_driver, 'punix:/tmp/fake', helper)
+
+        self.mock_get_node = mock.patch.object(
+            hash_ring_manager.HashRingManager,
+            'get_node', return_value=self.node_uuid).start()
+
+    @mock.patch.object(db_hash_ring, 'touch_node')
+    def test_notify(self, mock_touch_node):
+        self.idl.notify(self.fake_event, self.fake_row)
+
+        mock_touch_node.assert_called_once_with(self.node_uuid)
+        self.idl.notify_handler.notify.assert_called_once_with(
+            self.fake_event, self.fake_row, None)
+
+    @mock.patch.object(db_hash_ring, 'touch_node')
+    def test_notify_skip_touch_node(self, mock_touch_node):
+        # Set a time for last touch
+        self.idl._last_touch = timeutils.utcnow()
+        self.idl.notify(self.fake_event, self.fake_row)
+
+        # Assert that touch_node() wasn't called
+        self.assertFalse(mock_touch_node.called)
+        self.idl.notify_handler.notify.assert_called_once_with(
+            self.fake_event, self.fake_row, None)
+
+    @mock.patch.object(db_hash_ring, 'touch_node')
+    def test_notify_last_touch_expired(self, mock_touch_node):
+        # Set a time for last touch
+        self.idl._last_touch = timeutils.utcnow()
+
+        # Let's expire the touch node interval for the next utcnow()
+        with mock.patch.object(timeutils, 'utcnow') as mock_utcnow:
+            mock_utcnow.return_value = (
+                self.idl._last_touch + datetime.timedelta(
+                    seconds=ovn_const.HASH_RING_TOUCH_INTERVAL + 1))
+            self.idl.notify(self.fake_event, self.fake_row)
+
+        # Assert that touch_node() was invoked
+        mock_touch_node.assert_called_once_with(self.node_uuid)
+        self.idl.notify_handler.notify.assert_called_once_with(
+            self.fake_event, self.fake_row, None)
+
+    @mock.patch.object(ovsdb_monitor.LOG, 'exception')
+    @mock.patch.object(db_hash_ring, 'touch_node')
+    def test_notify_touch_node_exception(self, mock_touch_node, mock_log):
+        mock_touch_node.side_effect = Exception('BoOooOmmMmmMm')
+        self.idl.notify(self.fake_event, self.fake_row)
+
+        # Assert that in an eventual failure on touch_node() the event
+        # will continue to be processed by notify_handler.notify()
+        mock_touch_node.assert_called_once_with(self.node_uuid)
+        # Assert we are logging the exception
+        self.assertTrue(mock_log.called)
+        self.idl.notify_handler.notify.assert_called_once_with(
+            self.fake_event, self.fake_row, None)
+
+    def test_notify_different_node(self):
+        self.mock_get_node.return_value = 'different-node-uuid'
+        self.idl.notify('fake-event', self.fake_row)
+        # Assert that notify() wasn't called for a different node uuid
+        self.assertFalse(self.idl.notify_handler.notify.called)
+
+
 class TestPortBindingChassisUpdateEvent(base.TestCase):
     def setUp(self):
         super(TestPortBindingChassisUpdateEvent, self).setUp()
