@@ -66,7 +66,7 @@ ln -s ~/networking-ovn /opt/stack/networking-ovn
 # We need swap space to do any sort of scale testing with the Vagrant config.
 # Without this, we quickly run out of RAM and the kernel starts whacking things.
 sudo rm -f /swapfile1
-sudo dd if=/dev/zero of=/swapfile1 bs=1024 count=8388608
+sudo dd if=/dev/zero of=/swapfile1 bs=1024 count=2097152
 sudo chown root:root /swapfile1
 sudo chmod 0600 /swapfile1
 sudo mkswap /swapfile1
@@ -74,8 +74,14 @@ sudo swapon /swapfile1
 
 # Configure MTU on VM interfaces. Also requires manually configuring the same MTU on
 # the equivalent 'vboxnet' interfaces on the host.
-sudo ip link set dev eth1 mtu $MTU
-sudo ip link set dev eth2 mtu $MTU
+
+if ip a | grep enp0; then
+    sudo ip link set dev enp0s8 mtu $MTU
+    sudo ip link set dev enp0s9 mtu $MTU
+else
+    sudo ip link set dev eth1 mtu $MTU
+    sudo ip link set dev eth2 mtu $MTU
+fi
 
 # Migration setup
 sudo sh -c "echo \"$OVN_DB_IP $OVN_DB_NAME\" >> /etc/hosts"
@@ -91,6 +97,8 @@ chmod 600 ~/.ssh/id_rsa
 echo "Host *" >> ~/.ssh/config
 echo "    StrictHostKeyChecking no" >> ~/.ssh/config
 chmod 600 ~/.ssh/config
+sudo mkdir /root/.ssh
+chmod 700 /root/.ssh
 sudo cp ~vagrant/.ssh/id_rsa /root/.ssh
 sudo cp ~vagrant/.ssh/authorized_keys /root/.ssh
 sudo cp ~vagrant/.ssh/config /root/.ssh/config
